/**
 * Copyright (c) Members of the EGEE Collaboration. 2007-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *
 */
 
#ifndef _OPEN_CONF_FILE_H
# define _OPEN_CONF_FILE_H

/* For ssize_t (and size_t) */
#include <sys/types.h>


#ifndef TRUE 
   #define TRUE  1
#endif

#ifndef FALSE
   #define FALSE 0
#endif


#define BUFFER_SIZE_SCAS_CONFIG_FILE  100000 /* bytes */




typedef struct s_config_name_values
{
    char * name;
    char * value;
} * config_name_values;


char * trimmer (char * str);
int parse_scas_config_file (char *, ssize_t, config_name_values *, size_t *);
int open_scas_config_file (char * config_file_path, config_name_values * cnv, size_t * n_config_name_values);
int print_config_name_value (config_name_values cnv, size_t n_config_name_values);
int setup_scas_config_name_value (config_name_values cnv, size_t n_config_name_values);
char * get_scas_config_value_by_name (config_name_values cnv, size_t n_config_name_values, char * search_name);
int free_config_name_value (config_name_values * cnv, size_t n_config_name_values);


/*******************************************************************************************
Function:                  secure_fileopen
Description:               use this function to be sure that this file can't be hijacked
                           before you read or write to it.
                           Various permission checks will be performed.
Parameters:
    const char * filepath : The full path to the file to open.
    int modeAndPerms      : Specify the specific file type, permissions to the file and in writing 
                            situation, it will specify the file permission on writing.
                            file modes:  S_IFREG, S_IFLNK, and others
                                         S_IRUSR, S_IWUSR, S_IXUSR, 
                                         S_IRGRP, S_IWGRP, S_IXGRP, 
                                         S_IROTH, S_IWOTH, S_IXOTH
                                         S_ISUID, S_ISGID, S_ISVTX                                     
                            Example:     (S_IFREG|S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)
    int openflags         : Specifies for which purpose the file is going to be opened, read, write or both
                            Example:     O_RDONLY
Return:
                   => 0 :  Open File descriptor
                    < 0 :  Error code

*******************************************************************************************/
int secure_fileopen (char * filepath, int modeAndPerms, int openflags,
                     int isNoSymLink,
                     int realUIDmustOwnFile,
                     int realGIDmustOwnFile);


#endif /* _OPEN_CONF_FILE_H */
